/* Name: main.c
 * Project: 1-Key Keyboard
 * Author: Flip van den Berg - www.flipwork.nl
 * Creation Date: 2008-10-06
 * Based on AVR-USB drivers from Objective Developments - http://www.obdev.at/products/avrusb/index.html
 */


/* USB-Touch - a simple 1-Key Touch Keyboard with usb-hid-interface
* Author: Stefan Rehm - srehm@lavabit.com
* 19.08.2010
* Most of the code is taken from the "1-Key Keyboard" by Flip van den Berg - see above
*
* uC used: ATTiny2313 @ 16MHz, VCC=3.3V, powered from usb-Bus via 3.3V-regulator
*
* Connections:
*
* USB-D-         : PD2 (via 68R resistor)
* USB-D+         : PD3 (via 68R resistor, 1k5 resistor from USB-D+ to VCC)
*
* LED            : PB3 (to LED kathode, anode to VCC via resistor)
*
* Sensor         : PB1 (via 1k resistor) sensor plate approx. 1.5cmx3cm
* 47nF capacitor : between PB1 and PD6
*
* 
* The firmware outputs a pressed spacebar key as long as the sensor is touched
*
* Limitations / TODO:
*
* Calibration is only done once upon power-on
* Code space is very limited: only 2 Bytes left in the 2313's flash  
*/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>

#include "usbdrv.h"
#include "oddebug.h"



/* ------------------------------------------------------------------------- */

static uchar    reportBuffer[2];    /* buffer for HID reports */
static uchar    idleRate;           /* in 4 ms units */
static uchar    reportCount;		/* current report */



#define th 5
#define filtth 6

uint16_t i = 0;
uint16_t msum=0;
char last;
uint8_t state=0;
uint8_t x;

/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};
/* We use a simplifed keyboard report descriptor which does not support the
 * boot protocol. We don't allow setting status LEDs and we only allow one
 * simultaneous key press (except modifiers). We can therefore use short
 * 2 byte input reports.
 * The report descriptor has been created with usb.org's "HID Descriptor Tool"
 * which can be downloaded from http://www.usb.org/developers/hidpage/.
 * Redundant entries (such as LOGICAL_MINIMUM and USAGE_PAGE) have been omitted
 * for the second INPUT item.
 */



// sensor stuff
void discharge()
{
	DDRB |=  0b00000010;	// PB1 is output
	DDRD |=  0b01000000;	// PB6 is output
	PORTB &= 0b11111101;	// PB1=0	
	PORTD &= 0b10111111;	// PD6=0

}




uint16_t mess()
{
	uint16_t a=0;
	while (ACSR & (1 << ACO))

	{
		DDRD &=  0b10111111;	//PD6 is input / high-z
		PORTB |= 0b00000010;	//PPB=1
		DDRB &=  0b11111101;	// PB1 is input
		PORTB &= 0b11111101;	// PB1=0
		DDRD |=  0b01000000;	// PD6 is output / 0
		_delay_us(10);
		a++;
	}
	return a;
}

void calibrate()
{
	x=0;
	discharge();
	_delay_ms(1);
	for (x=0; x<8 ; x++)
	{
		msum+=mess();
		discharge();
		_delay_ms(1);
	}
	msum=msum >> 3;
	
}


int treshold()
{
	i=mess();
	if (i<(msum-th))
		return 1;
	else
		return 0;
		
}

void messs()
{
	uint8_t count=0;
	x=0;
	
	last = last << 1;
	if (treshold())
		last|=1;

	
	for (count=0; count<8; count++)
	{
		if (last & (1 << count)) x++;
	}
	
	// count no of 1s
	
	if ((state==0)& (x>filtth))
	{
		state=1;
	//	reportCount = 0; // start report 
	}
	if ((state==1)& (x<=filtth))
	{
		state=0;
		// reportCount = 0; // start report
	}
	
}

// end of sensor stuff



static void buildReport(void)
{
uchar key = 0; //if not changed by the if-statement below, then send an empty report
	
	reportCount = 0; // start report 
    
    if (state == 1){
		key = 44; // key = space
		} 
    


	reportCount++;
    reportBuffer[0] = 0;    /* no modifiers */
    reportBuffer[1] = key;
}





/* -------------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* -------------------------------------------------------------------------------- */

uchar	usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            buildReport();
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}




void    usbEventResetReady(void)
{

}

/* ------------------------------------------------------------------------- */
/* --------------------------------- main ---------------------------------- */
/* ------------------------------------------------------------------------- */

int main(void)
{



	ACSR|=(1 << ACBG);	// internal ref on
	

    DDRB=0xFF;
    DDRD=0b111110011;
	discharge();
	calibrate();
    

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    x = 0;
    while(--x){             /* fake USB disconnect for > 250 ms */

        _delay_ms(1);
    }
    usbDeviceConnect();




    sei();

    for(;;){    /* main event loop */
        // wdt_reset();
        usbPoll();
		messs();
		if (state)
        	PORTB &=0b11110111;
        else
        	
        	PORTB |=0b00001000;
        discharge();
		/* A USB keypress cycle is defined as a scancode being present in a report, and
		then absent from a later report. To press and release the Caps Lock key, instead of
		holding it down, we need to send the report with the Caps Lock scancode and
		then an empty report. */
		
		if(usbInterruptIsReady() && reportCount < 2){ /* we can send another key */
        	buildReport();
           	usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
        	}
  		_delay_ms(1);

	}
   	return 0;
}
